#ifndef _LCD_OLED_H_
#define _LCD_OLED_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <string.h>
#include "oled_fonts.h"

#define FILLED 1
#define EMPTY 0

#define LCD_OLED_I2C

#define LCD_OLED_SIZE_X 128
#define LCD_OLED_SIZE_Y 64

#if defined LCD_OLED_I2C
#include "i2c.h"
void lcd_oled_init(i2c_t *i2c_dev, uint8_t address); 

#elif defined LCD_OLED_SPI
#include "spi.h"
void lcd_oled_init(spi_t *spi_dev, pinNo_t pinNo, uint8_t address); 

#else
	#error "Please Define The communication Methode in ked/drivers/lcd_oled/lcd_oled.h"
#endif



void lcd_oled_enable();
void lcd_oled_disable();

void lcd_oled_reset_hard();
void lcd_oled_reset_soft();

uint8_t lcd_oled_is_ready();

void lcd_oled_sleep();
void lcd_oled_wake();

void lcd_oled_draw_pixel(uint16_t x, uint16_t y, uint8_t color); 
void lcd_oled_draw_line(uint16_t x, uint16_t y, uint16_t angle, uint16_t lenght, uint8_t color); 
void lcd_oled_draw_rectangle(uint16_t x, uint16_t y, uint16_t length, uint16_t width, uint8_t fill, uint8_t color);
void lcd_oled_draw_circle(uint16_t x, uint16_t y, uint16_t radius, uint8_t fill, uint8_t color); 
void lcd_oled_draw_circle_bresenham(int16_t xc, int16_t yc, uint8_t x, uint8_t y, uint8_t color);

void lcd_oled_scroll_right(uint16_t start, uint16_t stop);
void lcd_oled_scroll_left(uint16_t start, uint16_t stop);

void lcd_oled_scroll_up(uint16_t start, uint16_t stop);
void lcd_oled_scroll_down(uint16_t start, uint16_t stop);

void lcd_oled_set_font(uint8_t *font, uint8_t size, uint8_t spacing); 
void lcd_oled_print_char(uint16_t x, uint16_t y, uint8_t c, uint8_t color); 
void lcd_oled_print_text(uint16_t x, uint16_t y, uint8_t *text, uint16_t length, uint8_t color);
void lcd_oled_print_cursor(uint8_t blink); 

void lcd_oled_goto_pos(uint16_t x, uint16_t y);

uint8_t lcd_oled_display();
uint8_t lcd_oled_clear();

void lcd_oled_change_brightness(uint8_t brightness); 
void lcd_oled_change_contrast(uint8_t contrast); 

void lcd_oled_rotate(uint8_t angle); 
void lcd_oled_inverse(uint8_t yseNo); 
void lcd_oled_invert(uint8_t yseNo); 

#ifdef __cplusplus
}
#endif

#endif /* _LCD_OLED_H_ */
